import click

from .split import split_mainfest


@click.group(name="kyaml")
def kyaml():
    """Utility to manipulate Kubernetes YAML files."""


@kyaml.command()
@click.argument("infile", type=click.File("r"), default="-")
@click.option(
    "--output-dir",
    "-o",
    default="./output",
    type=click.Path(exists=True, file_okay=False, resolve_path=True),
)
def split(infile, output_dir):
    """Split a single multi-mainfest YAML file into multiple files."""
    split_mainfest(infile, output_dir)



"""
Utility to manipulate Kubernetes YAML files.

Note: This is entirely experimental at this point. Development may or may not
continue. All it does is split Kubernetes YAML files. It may grow new features
in the future. Public repo Coming Soon."""
from setuptools import find_packages, setup

dependencies = [
    "click",
    "pyyaml",
    "pathlib2 ; python_version<'3.4'",
]

setup(
    name="kyaml",
    version="0.1.3",
    # url='https://github.com/tylerdave/kyaml',
    license="BSD",
    author="Dave Forgac",
    author_email="tylerdave@tylerdave.com",
    description="Utility to manipulate Kubernetes YAML files.",
    long_description=__doc__,
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    zip_safe=False,
    platforms="any",
    install_requires=dependencies,
    entry_points={"console_scripts": ["kyaml = kyaml.cli:kyaml",],},
    classifiers=[
        # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
        "Development Status :: 2 - Pre-Alpha",
        # 'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        # 'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import os
import os.path
import yaml

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


def load_combined_manifest(input_file):
    with open(input_file) as infile:
        loaded = yaml.load_all(infile, Loader=Loader)
        documents = [doc for doc in loaded]
        return documents


def is_directory_empty(path):
    path = str(Path(path).resolve())
    if not os.path.isdir(path):
        raise Exception("{} is not a directory.".format(path))
    real_files = [f for f in os.listdir(path) if f != ".gitkeep"]
    return len(real_files) == 0


def write_yaml_document(document, filename, path="."):
    outfile_name = str(Path(path, filename).resolve())
    with open(outfile_name, "w") as outfile:
        yaml.dump(document, outfile, default_flow_style=False, Dumper=Dumper)


def split_mainfest(combined_file, output_directory):
    """Given an open file handle or stream containing multiple YAML documents
    split them into individual manifest files and store them in the output_directory."""
    if not is_directory_empty(output_directory):
        raise Exception("{} is not empty.".format(output_directory))

    loaded = yaml.load_all(combined_file, Loader=Loader)
    documents = [doc for doc in loaded]
    for doc in documents:
        unnamed_count = 0
        name = doc.get("metadata", {}).get("name", "").lower()
        if name == "":
            name = "UNNAMED-{:03}".format(unnamed_count)
            unnamed_count += 1
        kind = doc.get("kind", "UNKNOWN").lower()
        filename = "{}_{}.yaml".format(name, kind)
        write_yaml_document(doc, filename, path=output_directory)
